#!/bin/sh

# NAME:
#	scm - scm agnostic wrapper
#	
# DESCRIPTION:
#	This script hides the details of the SCM being used.
#	We determine the SCM by looking for clues like
#	CVS/Entries, .git etc.
#
#	We set SCM to the lower case name of the tool.
#	SCM_VAR is the uppercase version of SCM which allows us to
#	derive SCM_CMD and SCM_FLAGS as if need be SCM_*_FLAGS
#	(eg. SCM_DIFF_FLAGS).
#
#	For example if CVS/Entries exists and doing 'diff' we set:
#.nf
#	SCM=cvs
#	SCM_VAR=CVS
#	SCM_CMD=${CVS:-cvs}
#	SCM_FLAGS=${CVS_FLAGS}
#	SCM_DIFF_FLAGS=${CVS_DIFF_FLAGS}
#
#	and run:
#
#	$SCM_CMD $SCM_FLAGS diff $SCM_DIFF_FLAGS "$@"
#.fi
#
#	This allows caller to set CVS_FLAGS etc without needing to
#	know upfront which is being used.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>

# RCSid:
#	$Id: scm.sh,v 1.5 2016/04/10 23:57:52 sjg Exp $
#
#	@(#) Copyright (c) 2016 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

# we need this here, and it is not always available...
Which() {
    case "$1" in
    -*) t=$1; shift;;
    *) t=-x;;
    esac
    case "$1" in
    /*)	test $t $1 && echo $1;;
    *)
        # some shells cannot correctly handle `IFS`
	# in conjunction with the for loop.
	_dirs=`IFS=:; echo ${2:-$PATH}`
	for d in $_dirs
	do
	    test $t $d/$1 && { echo $d/$1; break; }
	done
	;;
    esac
}

# tr is insanely non-portable wrt char classes, so we need to
# spell out the alphabet. sed y/// would work too.
toUpper() {
    ${TR:-tr} abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ
}

if [ -z "$SCM" ]; then
    if [ -s CVS/Entries ]; then
        SCM=cvs
        CVS_DIFF_FLAGS=${CVS_DIFF_FLAGS:--up}
    elif [ -d .git ]; then
        SCM=git
    elif [ -d .hg ]; then
        SCM=hg
    elif [ -d .svn ]; then
        SCM=svn
    fi
fi
SCM_VAR=`echo $SCM | toUpper`
eval SCM_CMD=\${$SCM_VAR:-$SCM} SCM_FLAGS="\${${SCM_VAR}_FLAGS}"

scm() {
    op=$1; shift
    OP=`echo $op | toUpper`
    eval SCM_OP_FLAGS="\${${SCM_VAR}_${OP}_FLAGS}"
    $SCM_CMD $SCM_FLAGS $op $SCM_OP_FLAGS "$@"
}

case /$0.sh in
*/scm.*)
    while :
    do
        case "$1" in
        *=*) eval "$1"; shift;;
        *) break;;
        esac
    done
    scm "$@"
    ;;
esac

    
