:
# NAME:
#	scanps - find info in ps output
#
# SYNOPSIS:
#	. scanps.sh
#	scanPS [-d] "field" "value" "output"
#	scanps.sh [-d] "field" "value" "output"
#
# DESCRIPTION:
#	This module provides a function which can be used to extract
#	info from ps(1) output.	 For example, it can be used to get
#	the PID of a command, or the PPID of a process.
#
#	If "field" is 'CMD' then "value" is treated as regexp to look
#	for, otherwise we look for the line(s) that have
#	"field"=="value".  For each matching line, we output the field
#	named "out".
#
#	Examples:
#.nf
#
#	scanps.sh CMD scanps PID
#		Will print the PID of the scanps.sh process.
#		
#	scanps.sh PID $$ PPID
#		Will print the PPID of the process that ran scanps.sh
#
#.fi
#
#	Options:
#
#	-a	Print all lines from ps.
#	
#	-d	Causes the function to print out the full line that
#		matched.
#
#	-p "PS_OPTS"
#		Supply alternate options for ps(1).
#		By default we use 'ps -l' which is nice and portable,
#		but if looking for a command, the CMD field is very
#		short, and will often fail to match.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#	

#
# RCSid:
#	$Id: scanps.sh,v 1.5 2016/04/09 16:34:16 sjg Exp $
#
#	@(#) Copyright (c) 1994 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

scanPS() {
	doit='#'
	all='#'
	tpsf=${TMP:-/tmp}/.tpsf$$
	keep=
	
	while :
	do
		case "$1" in
		-p)	PS_OPTS="$2"; shift;;
		-d)	doit=;;
		-a)	all=;;
		-k)	keep=:;;
		*)	break;;
		esac
		shift
	done
	case "$1" in
	CMD)	re="$2";;
	*)	re=you_should_never_see_this;;
	esac
	out=${3:-PID}
	# do our best to get a result
	for o in ${PS_OPTS:--l} -xl -el
	do
		[ -s $tpsf ] && break
		${PS:-ps} $o > $tpsf 2>/dev/null
	done
	awk "
	BEGIN {
		first=1
	}
	/PID/ {
		if (first) {
			first=0
			f[\"CMD\"] = NF
			for (i = 1; i < NF; i++) {
				f[\$i] = i
			}
			$all print \$0
			next
		}
	}
	/$re/  {
		$all print \$0
		$doit print \"Found /$re/\", \$0
		print \$f[\"$out\"]
		next
	}
	{
		$all print \$0
		if (\$f[\"$1\"] == \"$2\") {
			$doit print \"Found $1==$2:\", \$0
			print \$f[\"$out\"]
			next
		}
	}" $tpsf
	$keep rm -f $tpsf
}

case $0 in
*scanps*)	scanPS $*;;
esac
